/***************************************************************************
                                  flurry.c
                             -------------------
    begin                : Fri Sep  8 2000
    copyright            : (C) 2000 by Stephan Uhlmann
    email                : su@codewizards.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <vga.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>



unsigned char* screen; 
unsigned char screen2[64000];

void swoosh()
{
 int i,c;
 
 for (i=0;i<64000;i++)
 {
  c=screen2[i];
  if (i-1 < 0) c=c+screen2[i]; else c=c+screen2[i-1];
  if (i+1 >= 64000) c=c+screen2[i]; else c=c+screen2[i+1];
  if (i-320 < 0) c=c+screen2[i]; else c=c+screen2[i-320];
  if (i+320 >= 64000) c=c+screen2[i]; else c=c+screen2[i+320];
  screen2[i]=(unsigned char)(c/5);
 }
}


void draw_dot(int x, int y)
{
 int i;
 if ((y>0) && (y<195) && (x>0) && (x<315))
 {
  i=y*320+x;
  memset(&screen2[i],127,4);
  memset(&screen2[i+320],127,4);
  memset(&screen2[i+640],127,4);
  memset(&screen2[i+960],127,4);
 }
}


int main()
{
 int i=0,j,k;
 int x,y,degree,xwave,ywave,size,dsize;
 unsigned long frames;
 time_t runtime;
 
 printf("flurry 1.0\nstephan uhlmann <su@codewizards.org>\n\n");

 printf("initializing svgalib...");
 i=vga_init();
 if (i!=0) { printf("ERROR.\n"); return 1; } else printf("ok\n");

 printf("setting graphicsmode...");
 i=vga_setmode(5);
 if (i!=0) { printf("ERROR.\n"); return 1; } else printf("ok\n");
 
 /* get video memory pointer */ 
 screen=vga_getgraphmem();
 

 /* generate palette */ 
 for (i=0;i<32;i++) vga_setpalette(i,0,0,(int)(10+i*22/32));
 for (i=32;i<64;i++) vga_setpalette(i,0,0,i);
 for (i=64;i<128;i++) vga_setpalette(i,i,i,63);
 
 degree=-180;
 xwave=3;ywave=1;
 size=200;dsize=-1;
 
 frames=0;
 time(&runtime);
 /* run it until key pressed */ 
 while (vga_getkey()==0)
 {
 
  /* cycle around */
  degree++;
  if (degree>180)
  {
   degree=-180;
   xwave=(int)((float)rand()/RAND_MAX*5+1);
   ywave=(int)((float)rand()/RAND_MAX*5+1);
  }

  /* scaling size */
  size=size+dsize;
  if (size<100) dsize=-dsize;
  if (size>200) dsize=-dsize;
  
  /* make some random noise in background */
  for (i=0;i<1000;i++)
  {
   j=(int) (64000.0*rand()/(RAND_MAX+1.0));
   k=(int) (48.0*rand()/(RAND_MAX+1.0));
   if (screen2[j]<k)
    screen2[j]=k;
  }
  
  /* draw the dots */
  x=(int)(sin(degree*xwave*3.141/180)*128)+160;
  y=(int)(cos(degree*ywave*3.141/180)*80)+100;
  draw_dot(x,y);

  x=(int)(x+cos(-degree*8*3.141/180)*-size/10);
  y=(int)(y+sin(-degree*8*3.141/180)*size/10);
  draw_dot(x,y);

  x=(int)(x+sin(degree*4*3.141/180)*size/10);
  y=(int)(y+cos(-degree*4*3.141/180)*size/10);
  draw_dot(x,y);

  x=(int)(x+sin(-degree*8*3.141/180)*-size/10);
  y=(int)(y+cos(degree*8*3.141/180)*-size/10);
  draw_dot(x,y);

  x=(int)(x+sin(degree*32*3.141/180)*-size/10);
  y=(int)(y+cos(degree*32*3.141/180)*-size/10);
  draw_dot(x,y);

  /* swoosh it (antalias) */
  swoosh();

  /* put doublebuffer to videomemory */
  memcpy(screen,screen2,64000);
  
  frames++;
 }

 /* print speed (frames per second) */
 runtime=time(NULL)-runtime;
 printf("%.1f fps\n",((float)frames)/runtime); 

 /* return to textmode */
 printf("returning to textmode...");
 i=vga_setmode(TEXT);
 if (i!=0) { printf("ERROR.\n"); return 1; } else printf("ok\n");

 return 0;
}

